/*
 * Metronom.cpp
 *
 * Created: 2014-01-11 17:13:09
 *  Author: Trojan
 */ 

#define F_CPU 4000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <math.h>
#include <util/delay.h>


volatile uint8_t  bpm = 90;
volatile uint16_t period;
volatile uint16_t miliseconds; 
volatile uint8_t miliseconds_c; 
volatile uint16_t timer_ms;
volatile uint8_t display;
uint8_t a = 1;
const uint8_t ocrv[101] = {255,242,228,215,203,192,181,171,161,152,144,136,128,121,114,108,102,96,91,85,81,76,72,68,64,60,57,54,51,48,45,43,40,38,36,34,32,30,29,27,25,24,23,21,20,19,18,17,16,15,14,13,13,12,11,11,10,10,9,8,8,8,7,7,6,6,6,5,5,5,4,4,4,4,4,3,3,3,3,3,3,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,0,0};
void display_switch();
void set_number(uint8_t);
void display_bmp();
void display_power();
void display_a();
int8_t Read2StepEncoder();
volatile uint8_t lampa = 0;
uint8_t digits[3] = {1,2,3};
enum impulsator {LEFT,RIGHT, NONE} tact;
enum states {LAMP, METRONOM_BPM, METRONOM_SLOPE} state = LAMP;
uint8_t read_coder=0;
uint8_t coder_state = 0b00;
uint8_t lamp_power = 99;
int8_t enc_delta;

int main(void)
{
	DDRB = 0xFF;
	PORTB |= 1<<PORTB3;
	_delay_ms(500);
	OCR2 = 255;
	TCCR2 = 0b01111100;
	DDRC = 0xFF; //entire PortC as output
	
	DDRD = 1<<DDRD0;
	//PORTC = 1<<PORTC0;
	TCCR1A = 0b00000000;
	TCCR1B = 0b00011010;
	ICR1 = 500;
	TIMSK = 1<<TICIE1;
	period = 60000/bpm;
	
	
	MCUCR = 0b0101;
	GICR |= 1<<INT0;
	
	sei();
	
	while(lamp_power >= 51)
	{
		if(display) lamp_power--;
		if(display) display_switch();
		display_power();
		tact = NONE;
		OCR2 = ocrv[lamp_power];
	}
	
	
    while(1)
    {
		
	while(state == LAMP)
	{
		if(TCCR2 != 0b01111100) TCCR2 = 0b01111100;
		if(tact == LEFT || tact == RIGHT)
		{
			if(tact == LEFT) lamp_power++;
			if(lamp_power==101) lamp_power = 100;
			if(tact == RIGHT) lamp_power--;
			if(lamp_power == 0xFF) lamp_power = 0;
			
			display_power();
			miliseconds = 0;
			tact = NONE;
			OCR2 = ocrv[lamp_power];
		}
				
		if(display) display_switch();
		
		if(bit_is_clear(PIND,1))
		{
			display_bmp();
			_delay_ms(400);
			state = METRONOM_BPM;
			miliseconds = 0;
		}
		
	}
	while(state == METRONOM_BPM)
	{	
		if(display) display_switch();
		

		
		if(tact == LEFT || tact == RIGHT)
		{
		 if(tact == LEFT) bpm++;
		 if(bpm==251) bpm = 1;
		 if(tact == RIGHT) bpm--;
		 if(bpm==0) bpm = 250;
			period = 60000/bpm;
			display_bmp();
			miliseconds = 0;
			tact = NONE;
		}
		
			if(bit_is_clear(PIND,1))
			{
				timer_ms = 0;
				while (bit_is_clear(PIND,1));
				display = 1; miliseconds = 0;
				if(timer_ms <30 && bit_is_set(PIND,1)) {}
				else if(timer_ms < 700)
				{state = METRONOM_SLOPE; display_a(); }
				else { state = LAMP; display_power(); display = 1;}
			}
	}
	
	while(state == METRONOM_SLOPE)
		{
			if(display) display_switch();
			
			if(tact == LEFT || tact == RIGHT)
			{
				if(tact == LEFT) a++;
				if(a==11) a=10;
				if(tact == RIGHT) a--;
				if(a==0xFF) a=0; 
				display_a();
				miliseconds = 0;
				tact = NONE;
			}
					
			if(bit_is_clear(PIND,1))
			{	
				timer_ms = 0;
				while (bit_is_clear(PIND,1));
				display = 1; miliseconds = 0;
				if(timer_ms <30 && bit_is_set(PIND,1)) {}
				else if(timer_ms < 700)
				{state = METRONOM_BPM; display_bmp(); }
				else { state = LAMP; display_power(); display = 1;}
			}
		}
    }
}

ISR(TIMER1_CAPT_vect )
{
	miliseconds++;
	miliseconds_c++;
	timer_ms++;
	if(miliseconds_c%4 == 0) {display = 1; read_coder = 1;}
	

	if(state != LAMP)
	{
		
		
	if (miliseconds == period) 
	{
		miliseconds = 0;
		if(a==0)
		{
			OCR2 = 0;
			TCCR2 = 0b01001100;
			PORTB &= 0b11110111;
			_delay_ms(10);
			PORTB |= 0b00001000;
			TCCR2 = 0b01111100;
		}
	}
	
	if(a!=0){
				uint16_t diff = period - miliseconds;
				if (miliseconds>diff)
				{

					if(diff<100*a) OCR2 = ocrv[diff/a];
				}
				else
				{
					uint16_t miliseconds_latch = miliseconds;
					if(miliseconds_latch<100*a) OCR2 = ocrv[miliseconds_latch/a];
				}
	}
				if (OCR2 == 0) TCCR2 = 0b01001100; else TCCR2 = 0b01111100;
	}
	
}

void display_switch()
{
				display = 0;
					if(lampa == 0)
					{
						lampa++;
						set_number(digits[0]);
						PORTC&= 0b11111101;
						PORTB&= 0b11011111;
						PORTC|= 1<<PORTC2;
						return;
					}
					if(lampa == 1)
					{
						set_number(digits[1]);
						lampa++;
						PORTC&= 0b11111011;
						PORTB&= 0b11011111;
						PORTC|= 1<<PORTC1;
						return;
					}
					if(lampa == 2)
					{
						set_number(digits[2]);
						lampa=0;
						PORTC&= 0b11111001;
						PORTB|= 1<<PORTB5;
						return;
					}		
}

void set_number(uint8_t number)
{
	PORTB |= ~(0b11101001);
	PORTC |= ~(0b11000110);
	PORTD |= ~(0b11111110);
	
	if(number >=20 && number <30) 
	{
		number = number - 20;
		PORTD &= 0b11111110;
	}

	switch (number)
	{
		case 0:
		PORTB &= 0b11111001;
		PORTC &= 0b11000110;
		break; 
		case 1:
		PORTB &= 0b11111011;
		PORTC &= 0b11101111;
		break;
		case 2:
		PORTB &= 0b11101101;
		PORTC &= 0b11001110;
		break;
		case 3:
		PORTB &= 0b11101001;
		PORTC &= 0b11001111;
		break;
		case 4:
		PORTB &= 0b11101011;
		PORTC &= 0b11100111;
		break;
		case 5:
		PORTB &= 0b11101001;
		PORTC &= 0b11010111;
		break;
		case 6:
		PORTB &= 0b11101001;
		PORTC &= 0b11010110;
		break;
		case 7:
		PORTB &= 0b11111011;
		PORTC &= 0b11001111;
		break;
		case 8:
		PORTB &= 0b11101001;
		PORTC &= 0b11000110;
		break;
		case 9:
		PORTB &= 0b11101001;
		PORTC &= 0b11000111;
		break;
		default:
		break;
	}
}

void display_bmp()
{
	digits[2] = bpm/100;
	if(digits[2]==0) digits[2] = 50;
	digits[1] = (bpm%100)/10;
	digits[0] = bpm%10 +20;
}

void display_power()
{
	uint8_t tmp = 100-lamp_power;
	digits[2] = tmp/100;
	if(digits[2]==0) digits[2] = 50;
	digits[1] = (tmp%100)/10;
	digits[0] = tmp%10 ;
}

void display_a()
{
	digits[2] = 50;
	digits[1] = (a/10)+20;
	digits[0] = a%10;
}



ISR (INT0_vect)
{
	_delay_us(500);
	uint8_t tmp = PIND;
	if(bit_is_set(tmp,3))
	{
		if(bit_is_set(tmp,2))
		tact = LEFT;
		else
		{
			tact = RIGHT;
		}
		
	}
	else
	{
		if(bit_is_set(tmp,2))
				{
					tact = RIGHT;
				}
		else
		tact = LEFT;
	}
}
